@@ -24,13 +24,13 @@ import java.util.Collection;
 import java.util.Collections;
 import java.util.Deque;
 import java.util.LinkedList;
+import java.util.concurrent.Callable;
 import java.util.concurrent.Executor;
-import java.util.concurrent.ExecutorService;
 import java.util.concurrent.locks.Condition;
 import java.util.concurrent.locks.ReentrantLock;
 
 class DefaultBuildOperationQueue<T extends BuildOperation> implements BuildOperationQueue<T> {
-    private enum State {
+    private enum QueueState {
         Working, Finishing, Cancelled, Done
     }
 
@@ -43,13 +43,14 @@ class DefaultBuildOperationQueue<T extends BuildOperation> implements BuildOpera
     // Lock protects the following state, using an intentionally simple locking strategy
     private final ReentrantLock lock = new ReentrantLock();
     private final Condition workAvailable = lock.newCondition();
-    private final Condition workDone = lock.newCondition();
-    private State state = State.Working;
-    private int workers;
+    private final Condition operationsComplete = lock.newCondition();
+    private QueueState queueState = QueueState.Working;
+    private int workerCount;
+    private int pendingOperations;
     private final Deque<T> workQueue = new LinkedList<T>();
     private final LinkedList<Throwable> failures = new LinkedList<Throwable>();
 
-    DefaultBuildOperationQueue(WorkerLeaseService workerLeases, ExecutorService executor, QueueWorker<T> queueWorker) {
+    DefaultBuildOperationQueue(WorkerLeaseService workerLeases, Executor executor, QueueWorker<T> queueWorker) {
         this.workerLeases = workerLeases;
         this.parentWorkerLease = workerLeases.getWorkerLease();
         this.executor = executor;
@@ -60,17 +61,17 @@ class DefaultBuildOperationQueue<T extends BuildOperation> implements BuildOpera
     public void add(final T operation) {
         lock.lock();
         try {
-            if (state == State.Done) {
+            if (queueState == QueueState.Done) {
                 throw new IllegalStateException("BuildOperationQueue cannot be reused once it has completed.");
             }
-            if (state == State.Cancelled) {
+            if (queueState == QueueState.Cancelled) {
                 return;
             }
             workQueue.add(operation);
+            pendingOperations++;
             workAvailable.signalAll();
-            if (workers == 0 || workers < workerLeases.getMaxWorkerCount()) {
+            if (workerCount == 0 || workerCount < workerLeases.getMaxWorkerCount()) {
                 // TODO This could be more efficient, so that we only start a worker when there are none idle _and_ there is a worker lease available
-                workers++;
                 executor.execute(new WorkerRunnable());
             }
         } finally {
@@ -82,10 +83,11 @@ class DefaultBuildOperationQueue<T extends BuildOperation> implements BuildOpera
     public void cancel() {
         lock.lock();
         try {
-            if (state == State.Cancelled || state == State.Done) {
+            if (queueState == QueueState.Cancelled || queueState == QueueState.Done) {
                 return;
             }
-            state = State.Cancelled;
+            queueState = QueueState.Cancelled;
+            completeOperations(workQueue.size());
             workQueue.clear();
             workAvailable.signalAll();
         } finally {
@@ -96,19 +98,35 @@ class DefaultBuildOperationQueue<T extends BuildOperation> implements BuildOpera
     public void waitForCompletion() throws MultipleBuildOperationFailures {
         lock.lock();
         try {
-            if (state == State.Done) {
+            if (queueState == QueueState.Done) {
                 throw new IllegalStateException("Cannot wait for completion more than once.");
             }
-            state = State.Finishing;
+            queueState = QueueState.Finishing;
             workAvailable.signalAll();
-            while (workers > 0) {
+        } finally {
+            lock.unlock();
+        }
+
+        // Use this thread to process any work - this should only actually process work if none of the
+        // submitted workers make it onto the executor thread pool.
+        try {
+            new WorkerRunnable().run();
+        } catch (Throwable t) {
+            addFailure(t);
+        }
+
+        lock.lock();
+        try {
+            // Wait for any work still running in other threads
+            while (pendingOperations > 0) {
                 try {
-                    workDone.await();
+                    operationsComplete.await();
                 } catch (InterruptedException e) {
                     throw UncheckedException.throwAsUncheckedException(e);
                 }
             }
-            state = State.Done;
+
+            queueState = QueueState.Done;
             if (!failures.isEmpty()) {
                 throw new MultipleBuildOperationFailures(getFailureMessage(failures), failures, logLocation);
             }
@@ -117,6 +135,25 @@ class DefaultBuildOperationQueue<T extends BuildOperation> implements BuildOpera
         }
     }
 
+    private void addFailure(Throwable failure) {
+        lock.lock();
+        try {
+            failures.add(failure);
+        } finally {
+            lock.unlock();
+        }
+    }
+
+    private void completeOperations(int count) {
+        lock.lock();
+        try {
+            pendingOperations = pendingOperations - count;
+            operationsComplete.signalAll();
+        } finally {
+            lock.unlock();
+        }
+    }
+
     @Override
     public void setLogLocation(String logLocation) {
         this.logLocation = logLocation;
@@ -142,7 +179,7 @@ class DefaultBuildOperationQueue<T extends BuildOperation> implements BuildOpera
         private T waitForNextOperation() {
             lock.lock();
             try {
-                while (state == State.Working && workQueue.isEmpty()) {
+                while (queueState == QueueState.Working && workQueue.isEmpty()) {
                     try {
                         workAvailable.await();
                     } catch (InterruptedException e) {
@@ -156,16 +193,24 @@ class DefaultBuildOperationQueue<T extends BuildOperation> implements BuildOpera
         }
 
         private void runBatch(final T firstOperation) {
-            workerLeases.withLocks(Collections.singleton(parentWorkerLease.createChild()), new Runnable() {
-                @Override
-                public void run() {
-                    T operation = firstOperation;
-                    while (operation != null) {
-                        runOperation(operation);
-                        operation = getNextOperation();
+            // We need to update pending count outside of withLocks() so that we don't have a race
+            // condition where the pending count is 0, but a child worker lease is still held when
+            // the parent lease is released.
+            completeOperations(
+                workerLeases.withLocks(Collections.singleton(parentWorkerLease.createChild()), new Callable<Integer>() {
+                    @Override
+                    public Integer call() throws Exception {
+                        int operationCount = 0;
+                        T operation = firstOperation;
+                        while (operation != null) {
+                            runOperation(operation);
+                            operationCount++;
+                            operation = getNextOperation();
+                        }
+                        return operationCount;
                     }
-                }
-            });
+                })
+            );
         }
 
         private T getNextOperation() {
@@ -177,7 +222,6 @@ class DefaultBuildOperationQueue<T extends BuildOperation> implements BuildOpera
             }
         }
 
-
         private void runOperation(T operation) {
             try {
                 queueWorker.execute(operation);
@@ -186,20 +230,10 @@ class DefaultBuildOperationQueue<T extends BuildOperation> implements BuildOpera
             }
         }
 
-        private void addFailure(Throwable failure) {
-            lock.lock();
-            try {
-                failures.add(failure);
-            } finally {
-                lock.unlock();
-            }
-        }
-
         private void shutDown() {
             lock.lock();
             try {
-                workers--;
-                workDone.signalAll();
+                workerCount--;
             } finally {
                 lock.unlock();
             }
