@@ -18,11 +18,14 @@ package org.gradle.api.internal.file.collections;
 import groovy.lang.Closure;
 import org.gradle.api.Buildable;
 import org.gradle.api.Task;
+import org.gradle.api.file.DirectoryVar;
 import org.gradle.api.file.FileCollection;
+import org.gradle.api.file.RegularFileVar;
 import org.gradle.api.internal.tasks.TaskDependencyResolveContext;
 import org.gradle.api.tasks.TaskOutputs;
 import org.gradle.internal.file.PathToFileResolver;
 
+import java.nio.file.Path;
 import java.util.concurrent.Callable;
 
 import static org.gradle.util.GUtil.uncheckedCall;
@@ -56,11 +59,12 @@ public class BuildDependenciesOnlyFileCollectionResolveContext implements FileCo
         } else if (element instanceof MinimalFileCollection && element instanceof Buildable) {
             taskContext.add(element);
         } else if (element instanceof Task) {
-            Task task = (Task) element;
-            taskContext.add(task);
+            taskContext.add(element);
         } else if (element instanceof TaskOutputs) {
             TaskOutputs outputs = (TaskOutputs) element;
             taskContext.add(outputs.getFiles());
+        } else if (element instanceof RegularFileVar || element instanceof DirectoryVar) {
+            taskContext.add(element);
         } else if (element instanceof Closure) {
             Closure closure = (Closure) element;
             Object closureResult = closure.call();
@@ -73,7 +77,8 @@ public class BuildDependenciesOnlyFileCollectionResolveContext implements FileCo
             if (callableResult != null) {
                 add(callableResult);
             }
-        } else if (element instanceof Iterable) {
+        } else if (element instanceof Iterable && !(element instanceof Path)) {
+            // Ignore Path
             Iterable<?> iterable = (Iterable) element;
             for (Object value : iterable) {
                 add(value);
@@ -87,4 +92,4 @@ public class BuildDependenciesOnlyFileCollectionResolveContext implements FileCo
         // Everything else assume has no dependencies
         return this;
     }
-}
\ No newline at end of file
+}
