@@ -18,13 +18,23 @@ package org.gradle.api.internal.provider;
 
 import org.gradle.api.provider.Provider;
 
+import javax.annotation.Nullable;
+
 public abstract class AbstractMappingProvider<OUT, IN> extends AbstractProvider<OUT> {
+    private final Class<OUT> type;
     private final Provider<? extends IN> provider;
 
-    public AbstractMappingProvider(Provider<? extends IN> provider) {
+    public AbstractMappingProvider(Class<OUT> type, Provider<? extends IN> provider) {
+        this.type = type;
         this.provider = provider;
     }
 
+    @Nullable
+    @Override
+    public Class<OUT> getType() {
+        return type;
+    }
+
     @Override
     public boolean isPresent() {
         return provider.isPresent();
