@@ -16,14 +16,16 @@
 
 package org.gradle.api.internal.tasks.execution;
 
+import com.google.common.collect.ImmutableList;
 import org.gradle.api.internal.TaskInternal;
 import org.gradle.api.internal.changedetection.TaskArtifactState;
+import org.gradle.api.internal.changedetection.rules.TaskUpToDateState;
 import org.gradle.api.internal.tasks.TaskExecuter;
 import org.gradle.api.internal.tasks.TaskExecutionContext;
 import org.gradle.api.internal.tasks.TaskExecutionOutcome;
 import org.gradle.api.internal.tasks.TaskStateInternal;
+import org.gradle.internal.time.Time;
 import org.gradle.internal.time.Timer;
-import org.gradle.internal.time.Timers;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -44,33 +46,21 @@ public class SkipUpToDateTaskExecuter implements TaskExecuter {
 
     public void execute(TaskInternal task, TaskStateInternal state, TaskExecutionContext context) {
         LOGGER.debug("Determining if {} is up-to-date", task);
-        Timer clock = Timers.startTimer();
+        Timer clock = Time.startTimer();
         TaskArtifactState taskArtifactState = context.getTaskArtifactState();
-        try {
-            List<String> messages = LOGGER.isInfoEnabled() ? new ArrayList<String>() : null;
-            if (taskArtifactState.isUpToDate(messages)) {
-                LOGGER.info("Skipping {} as it is up-to-date (took {}).", task, clock.getElapsed());
-                state.setOutcome(TaskExecutionOutcome.UP_TO_DATE);
-                return;
-            }
-            logOutOfDateMessages(messages, task, clock.getElapsed());
 
-            task.getOutputs().setHistory(taskArtifactState.getExecutionHistory());
-
-            taskArtifactState.beforeTask();
-            try {
-                executer.execute(task, state, context);
-                if (state.getFailure() == null) {
-                    taskArtifactState.afterTask();
-                }
-            } finally {
-                task.getOutputs().setHistory(null);
-            }
-        } finally {
-            taskArtifactState.finished();
+        List<String> messages = new ArrayList<String>(TaskUpToDateState.MAX_OUT_OF_DATE_MESSAGES);
+        if (taskArtifactState.isUpToDate(messages)) {
+            LOGGER.info("Skipping {} as it is up-to-date (took {}).", task, clock.getElapsed());
+            state.setOutcome(TaskExecutionOutcome.UP_TO_DATE);
+            context.setOriginBuildInvocationId(taskArtifactState.getOriginBuildInvocationId());
+            return;
         }
-    }
+        context.setUpToDateMessages(ImmutableList.copyOf(messages));
+        logOutOfDateMessages(messages, task, clock.getElapsed());
 
+        executer.execute(task, state, context);
+    }
 
     private void logOutOfDateMessages(List<String> messages, TaskInternal task, String took) {
         if (LOGGER.isInfoEnabled()) {
@@ -82,5 +72,4 @@ public class SkipUpToDateTaskExecuter implements TaskExecuter {
             LOGGER.info(formatter.toString());
         }
     }
-
 }
