@@ -16,8 +16,8 @@
 
 package org.gradle.initialization;
 
-import org.gradle.initialization.buildsrc.BuildSourceBuilder;
 import org.gradle.composite.internal.IncludedBuildFactory;
+import org.gradle.initialization.buildsrc.BuildSourceBuilder;
 import org.gradle.internal.composite.CompositeBuildSettingsLoader;
 import org.gradle.internal.composite.CompositeContextBuilder;
 
@@ -25,42 +25,46 @@ public class DefaultSettingsLoaderFactory implements SettingsLoaderFactory {
     private final ISettingsFinder settingsFinder;
     private final SettingsProcessor settingsProcessor;
     private final BuildSourceBuilder buildSourceBuilder;
-    private final BuildLoader buildLoader;
     private final CompositeContextBuilder compositeContextBuilder;
     private final IncludedBuildFactory includedBuildFactory;
 
     public DefaultSettingsLoaderFactory(ISettingsFinder settingsFinder, SettingsProcessor settingsProcessor, BuildSourceBuilder buildSourceBuilder,
-                                        BuildLoader buildLoader, CompositeContextBuilder compositeContextBuilder, IncludedBuildFactory includedBuildFactory) {
+                                        CompositeContextBuilder compositeContextBuilder, IncludedBuildFactory includedBuildFactory) {
         this.settingsFinder = settingsFinder;
         this.settingsProcessor = settingsProcessor;
         this.buildSourceBuilder = buildSourceBuilder;
-        this.buildLoader = buildLoader;
         this.compositeContextBuilder = compositeContextBuilder;
         this.includedBuildFactory = includedBuildFactory;
     }
 
     @Override
     public SettingsLoader forTopLevelBuild() {
-        return new NotifyingSettingsLoader(
-            new CompositeBuildSettingsLoader(
-                new DefaultSettingsLoader(
-                    settingsFinder,
-                    settingsProcessor,
-                    buildSourceBuilder
-                ),
-                compositeContextBuilder, includedBuildFactory
-            ),
-            buildLoader);
+        return notifyingSettingsLoader(compositeBuildSettingsLoader());
     }
 
     @Override
     public SettingsLoader forNestedBuild() {
-        return new NotifyingSettingsLoader(
-            new DefaultSettingsLoader(
-                settingsFinder,
-                settingsProcessor,
-                buildSourceBuilder
-            ),
-            buildLoader);
+        return notifyingSettingsLoader(defaultSettingsLoader());
     }
+
+    private SettingsLoader compositeBuildSettingsLoader() {
+        return new CompositeBuildSettingsLoader(
+            defaultSettingsLoader(),
+            compositeContextBuilder, includedBuildFactory
+        );
+    }
+
+    private SettingsLoader defaultSettingsLoader() {
+        return new DefaultSettingsLoader(
+            settingsFinder,
+            settingsProcessor,
+            buildSourceBuilder
+        );
+    }
+
+
+    private SettingsLoader notifyingSettingsLoader(SettingsLoader settingsLoader) {
+        return new NotifyingSettingsLoader(settingsLoader);
+    }
+
 }
