@@ -21,19 +21,22 @@ import org.gradle.api.artifacts.ResolvedArtifact;
 import org.gradle.api.artifacts.ResolvedModuleVersion;
 import org.gradle.api.artifacts.component.ComponentArtifactIdentifier;
 import org.gradle.api.internal.artifacts.ivyservice.dynamicversions.DefaultResolvedModuleVersion;
+import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.ResolvableArtifact;
 import org.gradle.api.tasks.TaskDependency;
 import org.gradle.internal.Factory;
+import org.gradle.internal.UncheckedException;
 import org.gradle.internal.component.model.IvyArtifactName;
 
 import java.io.File;
 
-public class DefaultResolvedArtifact implements ResolvedArtifact, Buildable {
+public class DefaultResolvedArtifact implements ResolvedArtifact, Buildable, ResolvableArtifact {
     private final ModuleVersionIdentifier owner;
     private final IvyArtifactName artifact;
     private final ComponentArtifactIdentifier artifactId;
     private final TaskDependency buildDependencies;
-    private Factory<File> artifactSource;
+    private volatile Factory<File> artifactSource;
     private File file;
+    private Throwable failure;
 
     public DefaultResolvedArtifact(ModuleVersionIdentifier owner, IvyArtifactName artifact, ComponentArtifactIdentifier artifactId, TaskDependency buildDependencies, Factory<File> artifactSource) {
         this.owner = owner;
@@ -87,27 +90,56 @@ public class DefaultResolvedArtifact implements ResolvedArtifact, Buildable {
         return owner.hashCode() ^ artifactId.hashCode();
     }
 
+    @Override
     public String getName() {
         return artifact.getName();
     }
 
+    @Override
     public String getType() {
         return artifact.getType();
     }
 
+    @Override
     public String getExtension() {
         return artifact.getExtension();
     }
 
+    @Override
     public String getClassifier() {
         return artifact.getClassifier();
     }
 
+    @Override
+    public ResolvedArtifact toPublicView() {
+        return this;
+    }
+
+    @Override
+    public boolean isResolved() {
+        synchronized (this) {
+            return file != null || failure != null;
+        }
+    }
+
+    @Override
     public File getFile() {
-        if (file == null) {
-            file = artifactSource.create();
-            artifactSource = null;
+        synchronized (this) {
+            if (file != null) {
+                return file;
+            }
+            if (failure != null) {
+                throw UncheckedException.throwAsUncheckedException(failure);
+            }
+            try {
+                file = artifactSource.create();
+                return file;
+            } catch (Throwable e) {
+                failure = e;
+                throw UncheckedException.throwAsUncheckedException(failure);
+            } finally {
+                artifactSource = null;
+            }
         }
-        return file;
     }
 }
