@@ -21,7 +21,6 @@ import org.gradle.api.Transformer;
 import org.gradle.api.artifacts.ModuleVersionSelector;
 import org.gradle.api.artifacts.component.ModuleComponentIdentifier;
 import org.gradle.api.artifacts.ComponentMetadataSupplier;
-import org.gradle.api.internal.artifacts.ivyservice.CacheLockingManager;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy.Version;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy.VersionParser;
 import org.gradle.internal.component.external.model.DefaultModuleComponentIdentifier;
@@ -59,13 +58,10 @@ public class DynamicVersionResolver implements DependencyToComponentIdResolver {
     private final List<String> repositoryNames = new ArrayList<String>();
     private final VersionedComponentChooser versionedComponentChooser;
     private final Transformer<ModuleComponentResolveMetadata, RepositoryChainModuleResolution> metaDataFactory;
-    private final CacheLockingManager cacheLockingManager;
 
-    public DynamicVersionResolver(VersionedComponentChooser versionedComponentChooser, Transformer<ModuleComponentResolveMetadata, RepositoryChainModuleResolution> metaDataFactory,
-                                  CacheLockingManager cacheLockingManager) {
+    public DynamicVersionResolver(VersionedComponentChooser versionedComponentChooser, Transformer<ModuleComponentResolveMetadata, RepositoryChainModuleResolution> metaDataFactory) {
         this.versionedComponentChooser = versionedComponentChooser;
         this.metaDataFactory = metaDataFactory;
-        this.cacheLockingManager = cacheLockingManager;
     }
 
     public void add(ModuleComponentRepository repository) {
@@ -205,12 +201,7 @@ public class DynamicVersionResolver implements DependencyToComponentIdResolver {
         }
 
         void resolve() {
-            cacheLockingManager.useCache(new Runnable() {
-                @Override
-                public void run() {
-                    versionListingResult.resolve();
-                }
-            });
+            versionListingResult.resolve();
             switch (versionListingResult.result.getState()) {
                 case Failed:
                     resolveResult.failed(versionListingResult.result.getFailure());
