@@ -17,11 +17,16 @@ package org.gradle.api.internal.artifacts.repositories;
 
 import groovy.lang.Closure;
 import org.gradle.api.Action;
+import org.gradle.api.ActionConfiguration;
 import org.gradle.api.InvalidUserDataException;
+import org.gradle.api.artifacts.ComponentMetadataSupplier;
 import org.gradle.api.artifacts.repositories.AuthenticationContainer;
 import org.gradle.api.artifacts.repositories.IvyArtifactRepository;
 import org.gradle.api.artifacts.repositories.IvyArtifactRepositoryMetaDataProvider;
 import org.gradle.api.artifacts.repositories.RepositoryLayout;
+import org.gradle.api.artifacts.repositories.RepositoryResourceAccessor;
+import org.gradle.api.internal.DefaultActionConfiguration;
+import org.gradle.api.internal.InstantiatorFactory;
 import org.gradle.api.internal.artifacts.ImmutableModuleIdentifierFactory;
 import org.gradle.api.internal.artifacts.ModuleVersionPublisher;
 import org.gradle.api.internal.artifacts.ivyservice.IvyContextManager;
@@ -32,16 +37,20 @@ import org.gradle.api.internal.artifacts.repositories.layout.GradleRepositoryLay
 import org.gradle.api.internal.artifacts.repositories.layout.IvyRepositoryLayout;
 import org.gradle.api.internal.artifacts.repositories.layout.MavenRepositoryLayout;
 import org.gradle.api.internal.artifacts.repositories.layout.ResolvedPattern;
+import org.gradle.api.internal.artifacts.repositories.resolver.ExternalRepositoryResourceAccessor;
 import org.gradle.api.internal.artifacts.repositories.resolver.IvyResolver;
 import org.gradle.api.internal.artifacts.repositories.resolver.PatternBasedResolver;
 import org.gradle.api.internal.artifacts.repositories.transport.RepositoryTransport;
 import org.gradle.api.internal.artifacts.repositories.transport.RepositoryTransportFactory;
 import org.gradle.api.internal.file.FileResolver;
+import org.gradle.internal.Factory;
 import org.gradle.internal.component.external.model.ModuleComponentArtifactIdentifier;
 import org.gradle.internal.component.external.model.ModuleComponentArtifactMetadata;
 import org.gradle.internal.reflect.Instantiator;
+import org.gradle.internal.resource.local.FileResourceRepository;
 import org.gradle.internal.resource.local.FileStore;
 import org.gradle.internal.resource.local.LocallyAvailableResourceFinder;
+import org.gradle.internal.service.DefaultServiceRegistry;
 import org.gradle.util.ConfigureUtil;
 
 import java.net.URI;
@@ -49,6 +58,14 @@ import java.util.LinkedHashSet;
 import java.util.Set;
 
 public class DefaultIvyArtifactRepository extends AbstractAuthenticationSupportedRepository implements IvyArtifactRepository, ResolutionAwareRepository, PublicationAwareRepository {
+    private final static Factory<ComponentMetadataSupplier> NO_METADATA_SUPPLIER = new Factory<ComponentMetadataSupplier>() {
+        @Override
+        public ComponentMetadataSupplier create() {
+            return null;
+        }
+    };
+    private static final Object[] NO_PARAMS = new Object[0];
+
     private Object baseUrl;
     private AbstractRepositoryLayout layout;
     private final AdditionalPatternsRepositoryLayout additionalPatternsLayout;
@@ -58,24 +75,36 @@ public class DefaultIvyArtifactRepository extends AbstractAuthenticationSupporte
     private final MetaDataProvider metaDataProvider;
     private final Instantiator instantiator;
     private final FileStore<ModuleComponentArtifactIdentifier> artifactFileStore;
+    private final FileStore<String> externalResourcesFileStore;
     private final IvyContextManager ivyContextManager;
     private final ImmutableModuleIdentifierFactory moduleIdentifierFactory;
+    private final InstantiatorFactory instantiatorFactory;
+    private Class<? extends ComponentMetadataSupplier> componentMetadataSupplierClass;
+    private Object[] componentMetadataSupplierParams;
+    private final FileResourceRepository fileResourceRepository;
 
     public DefaultIvyArtifactRepository(FileResolver fileResolver, RepositoryTransportFactory transportFactory,
-                                        LocallyAvailableResourceFinder<ModuleComponentArtifactMetadata> locallyAvailableResourceFinder, Instantiator instantiator,
-                                        FileStore<ModuleComponentArtifactIdentifier> artifactFileStore, AuthenticationContainer authenticationContainer,
+                                        LocallyAvailableResourceFinder<ModuleComponentArtifactMetadata> locallyAvailableResourceFinder,
+                                        FileStore<ModuleComponentArtifactIdentifier> artifactFileStore,
+                                        FileStore<String> externalResourcesFileStore,
+                                        AuthenticationContainer authenticationContainer,
                                         IvyContextManager ivyContextManager,
-                                        ImmutableModuleIdentifierFactory moduleIdentifierFactory) {
-        super(instantiator, authenticationContainer);
+                                        ImmutableModuleIdentifierFactory moduleIdentifierFactory,
+                                        InstantiatorFactory instantiatorFactory,
+                                        FileResourceRepository fileResourceRepository) {
+        super(instantiatorFactory.decorate(), authenticationContainer);
         this.fileResolver = fileResolver;
         this.transportFactory = transportFactory;
         this.locallyAvailableResourceFinder = locallyAvailableResourceFinder;
         this.artifactFileStore = artifactFileStore;
+        this.externalResourcesFileStore = externalResourcesFileStore;
         this.additionalPatternsLayout = new AdditionalPatternsRepositoryLayout(fileResolver);
         this.moduleIdentifierFactory = moduleIdentifierFactory;
+        this.instantiatorFactory = instantiatorFactory;
+        this.fileResourceRepository = fileResourceRepository;
         this.layout = new GradleRepositoryLayout();
         this.metaDataProvider = new MetaDataProvider();
-        this.instantiator = instantiator;
+        this.instantiator = instantiatorFactory.decorate();
         this.ivyContextManager = ivyContextManager;
     }
 
@@ -110,16 +139,52 @@ public class DefaultIvyArtifactRepository extends AbstractAuthenticationSupporte
     }
 
     private IvyResolver createResolver(RepositoryTransport transport) {
-        return new IvyResolver(
-                getName(), transport,
-                locallyAvailableResourceFinder,
-                metaDataProvider.dynamicResolve, artifactFileStore, ivyContextManager, moduleIdentifierFactory);
+        Instantiator instantiator = createDependencyInjectingInstantiator(transport);
+        return new IvyResolver(getName(), transport, locallyAvailableResourceFinder, metaDataProvider.dynamicResolve, artifactFileStore, ivyContextManager, moduleIdentifierFactory, createComponentMetadataSupplierFactory(instantiator), fileResourceRepository);
+    }
+
+    /**
+     * Creates a service registry giving access to the services we want to expose to rules and returns an instantiator that
+     * uses this service registry.
+     * @param transport the transport used to create the repository accessor
+     * @return a dependency injecting instantiator, aware of services we want to expose
+     */
+    private Instantiator createDependencyInjectingInstantiator(final RepositoryTransport transport) {
+        DefaultServiceRegistry registry = new DefaultServiceRegistry();
+        registry.addProvider(new Object() {
+            RepositoryResourceAccessor createResourceAccessor() {
+                return createRepositoryAccessor(transport);
+            }
+        });
+        return instantiatorFactory.inject(registry);
+    }
+
+    private RepositoryResourceAccessor createRepositoryAccessor(RepositoryTransport transport) {
+        return new ExternalRepositoryResourceAccessor(getUrl(), transport.getResourceAccessor(), externalResourcesFileStore);
+    }
+
+    private Factory<ComponentMetadataSupplier> createComponentMetadataSupplierFactory(final Instantiator instantiator) {
+        if (componentMetadataSupplierClass == null) {
+            return NO_METADATA_SUPPLIER;
+        }
+
+        return new Factory<ComponentMetadataSupplier>() {
+            @Override
+            public ComponentMetadataSupplier create() {
+                return instantiator.newInstance(componentMetadataSupplierClass, componentMetadataSupplierParams);
+            }
+        };
     }
 
     public URI getUrl() {
         return baseUrl == null ? null : fileResolver.resolveUri(baseUrl);
     }
 
+    @Override
+    public void setUrl(URI url) {
+        baseUrl = url;
+    }
+
     public void setUrl(Object url) {
         baseUrl = url;
     }
@@ -157,6 +222,19 @@ public class DefaultIvyArtifactRepository extends AbstractAuthenticationSupporte
         return metaDataProvider;
     }
 
+    public void setMetadataSupplier(Class<? extends ComponentMetadataSupplier> ruleClass) {
+        this.componentMetadataSupplierClass = ruleClass;
+        this.componentMetadataSupplierParams = NO_PARAMS;
+    }
+
+    @Override
+    public void setMetadataSupplier(Class<? extends ComponentMetadataSupplier> rule, Action<? super ActionConfiguration> configureAction) {
+        DefaultActionConfiguration configuration = new DefaultActionConfiguration();
+        configureAction.execute(configuration);
+        this.componentMetadataSupplierClass = rule;
+        this.componentMetadataSupplierParams = configuration.getParams();
+    }
+
     /**
      * Layout for applying additional patterns added via {@link #artifactPatterns} and {@link #ivyPatterns}.
      */
