@@ -18,7 +18,6 @@ package org.gradle.api.internal.attributes;
 
 import com.google.common.collect.Maps;
 import org.gradle.api.Action;
-import org.gradle.api.Nullable;
 import org.gradle.api.attributes.Attribute;
 import org.gradle.api.attributes.AttributeContainer;
 import org.gradle.api.attributes.AttributeMatchingStrategy;
@@ -31,6 +30,7 @@ import org.gradle.internal.component.model.AttributeSelectionSchema;
 import org.gradle.internal.component.model.ComponentAttributeMatcher;
 import org.gradle.internal.component.model.DefaultCompatibilityCheckResult;
 
+import javax.annotation.Nullable;
 import java.util.Collection;
 import java.util.List;
 import java.util.Map;
@@ -162,7 +162,7 @@ public class DefaultAttributesSchema implements AttributesSchemaInternal, Attrib
         }
 
         @Override
-        public void disambiguate(Attribute<?> attribute, Object requested, MultipleCandidatesResult<Object> result) {
+        public void disambiguate(Attribute<?> attribute, MultipleCandidatesResult<Object> result) {
             DisambiguationRule<Object> rules = disambiguationRules(attribute);
             rules.execute(result);
             if (result.hasResult()) {
@@ -175,6 +175,7 @@ public class DefaultAttributesSchema implements AttributesSchemaInternal, Attrib
                 return;
             }
 
+            Object requested = result.getConsumerValue();
             if (requested != null && result.getCandidateValues().contains(requested)) {
                 result.closestMatch(requested);
                 return;
