@@ -17,6 +17,7 @@
 package org.gradle.util;
 
 import net.jcip.annotations.ThreadSafe;
+import org.apache.commons.lang.StringUtils;
 import org.gradle.internal.Factory;
 import org.gradle.internal.featurelifecycle.DeprecatedFeatureUsage;
 import org.gradle.internal.featurelifecycle.LoggingDeprecatedFeatureHandler;
@@ -211,9 +212,13 @@ public class SingleMessageLogger {
         }
     }
 
-    public static void nagUserOfDeprecatedThing(String thing) {
+    public static void nagUserOfDeprecatedThing(String thing, String explanation) {
         if (isEnabled()) {
-            nagUserOfDeprecated(String.format("%s. This", thing));
+            if (StringUtils.isEmpty(explanation)) {
+                nagUserWith(String.format("%s. This %s.", thing, getDeprecationMessage()));
+            } else {
+                nagUserWith(String.format("%s. This %s. %s.", thing, getDeprecationMessage(), explanation));
+            }
         }
     }
 
