@@ -17,6 +17,8 @@
 package org.gradle.play.internal.twirl;
 
 import org.gradle.language.twirl.TwirlImports;
+import org.gradle.language.twirl.TwirlTemplateFormat;
+import org.gradle.language.twirl.internal.DefaultTwirlTemplateFormat;
 import org.gradle.scala.internal.reflect.ScalaCodecMapper;
 import org.gradle.scala.internal.reflect.ScalaMethod;
 import org.gradle.scala.internal.reflect.ScalaReflectionUtil;
@@ -24,35 +26,36 @@ import org.gradle.scala.internal.reflect.ScalaReflectionUtil;
 import java.io.File;
 import java.lang.reflect.InvocationTargetException;
 import java.util.Arrays;
+import java.util.Collection;
+import java.util.Collections;
+import java.util.List;
 
-class TwirlCompilerAdapterV10X implements VersionedTwirlCompilerAdapter {
+class TwirlCompilerAdapterV10X extends VersionedTwirlCompilerAdapter {
     private static final Iterable<String> SHARED_PACKAGES = Arrays.asList("play.twirl.compiler", "scala.io"); //scala.io is for Codec which is a parameter to twirl
 
     // Based on https://github.com/playframework/playframework/blob/2.4.0/framework/src/build-link/src/main/java/play/TemplateImports.java
-    private static final String DEFAULT_JAVA_IMPORTS =
-              "import models._;"
-            + "import controllers._;"
-            + "import play.api.templates.PlayMagic._;"
-            + "import java.lang._;"
-            + "import java.util._;"
-            + "import scala.collection.JavaConversions._;"
-            + "import scala.collection.JavaConverters._;"
-            + "import play.api.i18n._;"
-            + "import play.core.j.PlayMagicForJava._;"
-            + "import play.mvc._;"
-            + "import play.data._;"
-            + "import play.api.data.Field;"
-            + "import play.mvc.Http.Context.Implicit._;"
-            + "import views.html._;";
+    private static final Collection<String> DEFAULT_JAVA_IMPORTS = Arrays.asList(
+        "models._",
+        "controllers._",
+        "play.api.templates.PlayMagic._",
+        "java.lang._",
+        "java.util._",
+        "scala.collection.JavaConversions._",
+        "scala.collection.JavaConverters._",
+        "play.api.i18n._",
+        "play.core.j.PlayMagicForJava._",
+        "play.mvc._",
+        "play.data._",
+        "play.api.data.Field",
+        "play.mvc.Http.Context.Implicit._");
 
-    private static final String DEFAULT_SCALA_IMPORTS =
-              "import models._;"
-            + "import controllers._;"
-            + "import play.api.templates.PlayMagic._;"
-            + "import play.api.i18n._;"
-            + "import play.api.mvc._;"
-            + "import play.api.data._;"
-            + "import views.html._;";
+    private static final Collection<String> DEFAULT_SCALA_IMPORTS = Arrays.asList(
+        "models._",
+        "controllers._",
+        "play.api.templates.PlayMagic._",
+        "play.api.i18n._",
+        "play.api.mvc._",
+        "play.api.data._");
 
     private final String scalaVersion;
     private final String twirlVersion;
@@ -80,13 +83,19 @@ class TwirlCompilerAdapterV10X implements VersionedTwirlCompilerAdapter {
     }
 
     @Override
-    public Object[] createCompileParameters(ClassLoader cl, File file, File sourceDirectory, File destinationDirectory, TwirlImports defaultImports) throws ClassNotFoundException, NoSuchMethodException, InvocationTargetException, InstantiationException, IllegalAccessException {
+    public Object[] createCompileParameters(ClassLoader cl, final File file, File sourceDirectory, File destinationDirectory, TwirlImports defaultImports, TwirlTemplateFormat templateFormat, List<String> additionalImports) throws ClassNotFoundException, NoSuchMethodException, InvocationTargetException, InstantiationException, IllegalAccessException {
+        final Collection<String> defaultTwirlImports;
+        if (defaultImports == TwirlImports.JAVA) {
+            defaultTwirlImports = DEFAULT_JAVA_IMPORTS;
+        } else {
+            defaultTwirlImports = DEFAULT_SCALA_IMPORTS;
+        }
         return new Object[] {
                 file,
                 sourceDirectory,
                 destinationDirectory,
-                "play.twirl.api.HtmlFormat",
-                defaultImports == TwirlImports.JAVA ? DEFAULT_JAVA_IMPORTS : DEFAULT_SCALA_IMPORTS,
+                templateFormat.getFormatType(),
+                getImportsFor(templateFormat, defaultTwirlImports, additionalImports),
                 ScalaCodecMapper.create(cl, "UTF-8"),
                 isInclusiveDots(),
                 isUseOldParser()
@@ -110,4 +119,14 @@ class TwirlCompilerAdapterV10X implements VersionedTwirlCompilerAdapter {
     public String getDependencyNotation() {
         return "com.typesafe.play:twirl-compiler_" + scalaVersion + ":" + twirlVersion;
     }
+
+    @Override
+    public Collection<TwirlTemplateFormat> getDefaultTemplateFormats() {
+        return Arrays.<TwirlTemplateFormat>asList(
+            new DefaultTwirlTemplateFormat("html", "play.twirl.api.HtmlFormat", Collections.singleton("views.html._")),
+            new DefaultTwirlTemplateFormat("txt", "play.twirl.api.TxtFormat", Collections.singleton("views.txt._")),
+            new DefaultTwirlTemplateFormat("xml", "play.twirl.api.XmlFormat", Collections.singleton("views.xml._")),
+            new DefaultTwirlTemplateFormat("js", "play.twirl.api.JavaScriptFormat", Collections.singleton("views.js._"))
+        );
+    }
 }
